/*
 *  Copyright (C) 2011 Stephen F. Booth <me@sbooth.org>
 *  Copyright (C) 2012, 2013, 2014 Feisty Dog, LLC
 *  All Rights Reserved
 */

#import <Cocoa/Cocoa.h>

@protocol DecibelAppMethods;

// ========================================
// The protocol that must be implemented by Decibel plugins
// ========================================
@protocol DecibelPlugInMethods <NSObject>

@required

// Create the plugin
- (id) initWithDecibelApp:(NSObject <DecibelAppMethods> *)dBApp;

@optional

// This message is sent immediately before initWithDecibelApp: is called
+ (void) plugInWillLoad;

// This message is sent immediately before -dealloc is called on this instance
- (void) plugInWillUnload;

// Used to configure the plug-in, if -allowsUserConfiguration returns YES
// Plug-ins should not store their defaults in Decibel's preference domain, but instead use 
// CFPreferencesSetAppValue, passing the bundle identifier for applicationID
- (void) showConfigurationSheetForWindow:(NSWindow *)window;

// A popover for the plug-in's content
- (NSPopover *) popover;

// Should return a 22 x 22 pixel image used for the button controlling the plug-in's popover
- (NSImage *) iconForPopover;

// Should return the tool tip to use for the button controlling the plug-in's popover
- (NSString *) toolTipForPopover;

@end
