/*
 *  Copyright (C) 2011, 2012 Stephen F. Booth <me@sbooth.org>
 *  All Rights Reserved
 */

#import <Cocoa/Cocoa.h>

@protocol PlaylistEntryMethods;

// ========================================
// Notification Names
// ========================================
extern NSString * const		PlaybackDidStartNotification;
extern NSString * const		PlaybackDidStopNotification;
extern NSString * const		PlaybackDidPauseNotification;
extern NSString * const		PlaybackDidResumeNotification;
extern NSString * const		PlaybackDidCompleteNotification;
extern NSString * const		PlaybackWasSkippedNotification;

// ========================================
// Notification Keys
// ========================================
extern NSString * const		PlaylistEntryObjectKey;			// NSObject <PlaylistEntryMethods> *

// ========================================
// File addition option keys
// ========================================
extern NSString * const		SkipCueFilesFileAdditionOptionKey;
extern NSString * const		SkipM3UFilesFileAdditionOptionKey;

// ========================================
// Enums
// ========================================
enum ShuffleMode {
	ShuffleOff			= 0,
	ShuffleByTrack		= 1,
	ShuffleByAlbum		= 2,
	ShuffleByArtist		= 3
};

enum RepeatMode {
	RepeatOff			= 0,
	RepeatTrack			= 1,
	RepeatAlbum			= 2,
	RepeatArtist		= 3,
	RepeatAll			= 4
};

// ========================================
// The interface presented by Decibel's player to plugins
// ========================================
@protocol PlayerMethods

@property (readonly, strong) NSObject <PlaylistEntryMethods> * nowPlaying;

// ========================================
// State
- (BOOL) isPlaying;
- (BOOL) isShuffling;
- (BOOL) isRepeating;

- (enum ShuffleMode) shuffleMode;
- (enum RepeatMode) repeatMode;

// ========================================
// Availability
- (BOOL) canPlay;
- (BOOL) canSeek;
- (BOOL) canPlayPreviousTrack;
- (BOOL) canPlayNextTrackTrack;

// ========================================
// Action methods
- (IBAction) play:(id)sender;
- (IBAction) pause:(id)sender;
- (IBAction) stop:(id)sender;
- (IBAction) playPause:(id)sender;

- (IBAction) seekForward:(id)sender;
- (IBAction) seekBackward:(id)sender;

- (IBAction) playPreviousTrack:(id)sender;
- (IBAction) playNextTrack:(id)sender;

- (IBAction) increaseDeviceVolume:(id)sender;
- (IBAction) decreaseDeviceVolume:(id)sender;

- (IBAction) increaseDigitalVolume:(id)sender;
- (IBAction) decreaseDigitalVolume:(id)sender;

- (IBAction) clearPlaylist:(id)sender;
- (IBAction) scramblePlaylist:(id)sender;

- (IBAction) setShuffle:(id)sender;
- (IBAction) setRepeat:(id)sender;

// ========================================
// Playlist management
- (BOOL) canSelectPreviousTrack;
- (BOOL) canSelectNextTrack;
- (BOOL) canRemoveSelectedTracks;

- (IBAction) selectPreviousTrack:(id)sender;
- (IBAction) selectNextTrack:(id)sender;

- (IBAction) playSelection:(id)sender;
- (IBAction) removeSelectedTracks:(id)sender;

- (NSArray *) tracks;

- (NSArray *) selectedTracks;
- (void) selectTracks:(NSArray *)tracks;

// ========================================
// Playback position
- (SInt64) currentFrame;
- (SInt64) totalFrames;

- (CFTimeInterval) currentTime;
- (CFTimeInterval) totalTime;

- (double) currentPosition;

// ========================================
// Seeking
- (BOOL) seekToFrame:(SInt64)frame;
- (BOOL) seekToTime:(CFTimeInterval)timeInSeconds;
- (BOOL) seekToPosition:(double)position;

// ========================================
// Volume controls
- (float) deviceVolume;
- (void) setDeviceVolume:(float)volume;			// Linear, in range [0, 1]

- (double) digitalVolume;
- (void) setDigitalVolume:(double)volume;		// Linear, in range [0, 1]

- (BOOL) canAdjustDeviceVolume;
- (BOOL) canAdjustDigitalVolume;

// ========================================
// Track addition
- (void) addFile:(NSString *)filename;
- (void) addFile:(NSString *)filename options:(NSDictionary *)options;
- (void) addFiles:(NSArray *)filenames;
- (void) addFiles:(NSArray *)filenames options:(NSDictionary *)options;
- (void) addFilesAndPlay:(NSArray *)filenames;
- (void) addFilesAndPlay:(NSArray *)filenames options:(NSDictionary *)options;

- (void) addURL:(NSURL *)URL;
- (void) addURL:(NSURL *)URL options:(NSDictionary *)options;
- (void) addURLs:(NSArray *)URLs;
- (void) addURLs:(NSArray *)URLs options:(NSDictionary *)options;
- (void) addURLsAndPlay:(NSArray *)URLs;
- (void) addURLsAndPlay:(NSArray *)URLs options:(NSDictionary *)options;

- (void) insertURL:(NSURL *)URL atRowIndex:(NSUInteger)rowIndex;
- (void) insertURL:(NSURL *)URL atRowIndex:(NSUInteger)rowIndex options:(NSDictionary *)options;
- (void) insertURLs:(NSArray *)URLs atRowIndex:(NSUInteger)rowIndex;
- (void) insertURLs:(NSArray *)URLs atRowIndex:(NSUInteger)rowIndex options:(NSDictionary *)options;
- (void) insertURLs:(NSArray *)URLs atRowIndex:(NSUInteger)rowIndex play:(BOOL)play;
- (void) insertURLs:(NSArray *)URLs atRowIndex:(NSUInteger)rowIndex play:(BOOL)play options:(NSDictionary *)options;

- (void) playFile:(NSString *)filename;
- (void) playURL:(NSURL *)URL;

- (BOOL) playTrackAtIndex:(NSUInteger)trackIndex;
- (BOOL) playTrack:(NSObject <PlaylistEntryMethods> *)track;

@end
