/*
 *  Copyright (C) 2011, 2012 Stephen F. Booth <me@sbooth.org>
 *  Copyright (C) 2012, 2013, 2014 Feisty Dog, LLC
 *  All Rights Reserved
 */

#import <Cocoa/Cocoa.h>

// ========================================
// The interface representing a track presented by Decibel to plugins
// ========================================
@protocol PlaylistEntryMethods

// ========================================
// Location information
@property (copy) NSURL * URL;
@property (retain) __attribute__((NSObject)) CFUUIDRef UUID;

@property (readonly) BOOL resourceIsAccessible;

// ========================================
// Set if this track is part of a larger file
@property (readonly, copy) NSNumber * startMSF;
@property (readonly, copy) NSNumber * endMSF;

// ========================================
// File format information
@property (readonly, copy) NSString * formatName;
@property (readonly, copy) NSNumber * duration;
@property (readonly, copy) NSNumber * sampleRate;
@property (readonly, copy) NSNumber * totalFrames;
@property (readonly, copy) NSNumber * bitsPerChannel;
@property (readonly, copy) NSNumber * channelsPerFrame;
@property (readonly, copy) NSNumber * bitrate;

// ========================================
// File metadata information
// Note: Setters do NOT save metadata back to the original file
@property (copy) NSString * title;
@property (copy) NSString * artist;
@property (copy) NSString * albumTitle;
@property (copy) NSString * albumArtist;
@property (copy) NSNumber * trackNumber;
@property (copy) NSNumber * trackTotal;
@property (copy) NSNumber * discNumber;
@property (copy) NSNumber * discTotal;
@property (copy) NSNumber * rating;
@property (copy) NSNumber * compilation;
@property (copy) NSString * genre;
@property (copy) NSString * composer;
@property (copy) NSString * releaseDate;
@property (copy) NSString * ISRC;
@property (copy) NSString * MCN;
@property (copy) NSString * musicBrainzAlbumID;
@property (copy) NSString * musicBrainzTrackID;

// ========================================
// Replay gain
@property (copy) NSNumber * replayGainTrackGain;
@property (copy) NSNumber * replayGainTrackPeak;
@property (copy) NSNumber * replayGainAlbumGain;
@property (copy) NSNumber * replayGainAlbumPeak;

// ========================================
// Album art
@property (copy) NSImage * frontCoverArt;

@end
